function createChild(super, child)
    setmetatable(child, { __index = super })
    return child
end

function createStyle(font, color, style)
    for k, v in pairs(font) do
        style[k] = v
    end
    for k, v in pairs(color) do
        style[k] = v
    end
    return style
end

local function hasValue(set, value)
    for k, v in pairs(set) do
        if v == value then
            return true
        end
    end
    return false
end

editor = {}
editor.settings = {}

--兼容原来的旧配置
editSettings = editor.settings

editor.settings.eol = false

editor.settings.eolMode = 2
editor.settings.lineNumber = true

editor.settings.useVolumeKey = false

editor.settings.useScaleGesture = false

editor.settings.whitespace = {}
--SCWS_
editor.settings.whitespace.flags = NmmStyle.SCWS_INVISIBLE
editor.settings.whitespace.size = 4
editor.settings.bracesCheck = true

editor.settings.tab = {}
--SCTD_
editor.settings.tab.drawMode = NmmStyle.SCTD_LONGARROW
editor.settings.tab.size = 4

editor.settings.wrap = {}
--SC_WRAP_
editor.settings.wrap.style = NmmStyle.SC_WRAP_NONE

--If the buffer size is greater than editor.settings.wrap.maxsize, it is forced to WRAP_NONE
--8M
editor.settings.wrap.maxsize = 6 * 1024 * 1024

--SC_WRAPVISUALFLAG_
editor.settings.wrap.visualFlags = NmmStyle.SC_WRAPVISUALFLAG_NONE

--auto completion
editor.settings.autoCompletion = {}

editor.settings.autoCompletion.enable = true;

editor.settings.autoCompletion.maxsize = 1 * 1024 * 1024

editor.settings.caret = {}
--CARETSTYLE_
editor.settings.caret.style = NmmStyle.CARETSTYLE_LINE
editor.settings.caret.lineVisible = true

editor.settings.caret.blink = true

editor.settings.selection = {}
editor.settings.selection.usefore = true


editor.settings.indent = {}
--Indentation
editor.settings.indent.tabsize = 4

editor.settings.indent.size = 4
editor.settings.indent.auto = false
editor.settings.indent.usetabs = false
editor.settings.indent.backspaceUnindents = false

--indentation start word sets
editor.settings.indent.starts = { "{", "(", ":" }
--indentation end word sets
editor.settings.indent.ends = { "}" }
--SC_IV_
editor.settings.indent.guides = NmmStyle.SC_IV_NONE

--SC_CACHE_
editor.settings.cacheLayout = NmmStyle.SC_CACHE_NONE

--SC_IDLESTYLING_
editor.settings.idleStyling = NmmStyle.SC_IDLESTYLING_TOVISIBLE

editor.settings.fold = {}
--# Folding
--# enable folding , and show lines below when collapsed.
editor.settings.fold.enable = false
--arrow 0
--plus 1
--circle 2
--square 3
editor.settings.fold.symbols = 3

--SC_FOLDFLAG_
editor.settings.fold.flags = NmmStyle.SC_FOLDFLAG_LINEAFTER_CONTRACTED
editor.settings.fold.comment = true
--update settings from preference
nmmStyle:updateEditorSettings(editor.settings)

function updateBaseFont(baseFont)
    nmmStyle:updateBaseFont(baseFont)
end

--Only load resources in the assets directory. Don't use loadfile function
local schemes = load(nmmStyle:readAssets("nmm/colorschemes.lua"))()
local schemeModule = "schemes/light.lua"
if colorScheme == nil or colorScheme == "" then
    local colorScheme = nmmStyle:getColorScheme()
    if schemes[colorScheme] ~= nil then
        schemeModule = schemes[colorScheme]
    end
else
    if schemes[colorScheme] ~= nil then
        schemeModule = schemes[colorScheme]
    end
end

--兼容
nmm = nmmStyle

--print(colorScheme)
fonts, colors = dofile(schemeModule)()


characters = {}
characters.alpha = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
characters.numeric = "0123456789"
characters.accented = ""

wordCharacters = characters.alpha .. characters.numeric .. "_"

local globalStyles = {}
globalStyles[NmmStyle.STYLE_DEFAULT] = createStyle(fonts.base, colors.textDefault, {})
globalStyles[NmmStyle.STYLE_LINENUMBER] = createStyle(fonts.base, colors.lineNumber, {})
globalStyles[NmmStyle.STYLE_BRACELIGHT] = createStyle(fonts.base, colors.braceMatchHighlight, { bold = true })
globalStyles[NmmStyle.STYLE_INDENTGUIDE] = createStyle(fonts.base, colors.indentGuides, {})

globalStyles[NmmStyle.STYLE_FOLDDISPLAYTEXT] = createStyle(fonts.base, colors.textComment, {})



--load program language config


local function matchLexer()
    local lexers = load(nmmStyle:readAssets("nmm/lexers.lua"))()
    --if set language
    for i, v in ipairs(lexers) do
        if v.lang == language then
            return v
        end
    end

    return {
        extensions = { ".txt", ".log", ".lst", ".diz", ".nfo" },
        lang = "text",
        lexername = "text",
        conf = "style/text.lua"
    }
end

local lexer = matchLexer()


local keywords, style, properties = dofile(lexer.conf)()

if editor.settings.fold.enable then
    properties["fold"] = "1"
else
    properties["fold"] = "0"
end
if editor.settings.fold.comment then
    properties["fold.comment"] = "1"
else
    properties["fold.comment"] = "0"
end



--load config

nmmStyle:setLexerLanguage(lexer.lexername)
--set keywords
for i = 1, #keywords do
    nmmStyle:setKeyWords(i - 1, keywords[i])
end
--lexer properties
for k, v in pairs(properties) do
    nmmStyle:setProperty(k, v)
end

--style
nmmStyle:setOneStyle(NmmStyle.STYLE_DEFAULT, globalStyles[NmmStyle.STYLE_DEFAULT])
nmmStyle:styleClearAll()
for k, v in pairs(globalStyles) do
    nmmStyle:setOneStyle(k, v)
end
for k, v in pairs(style) do
    nmmStyle:setOneStyle(k, v)
end

nmmStyle:setWordChars(wordCharacters)

nmmStyle:setIdleStyling(editor.settings.idleStyling)

--tab
nmmStyle:setTabWidth(editor.settings.indent.tabsize)
nmmStyle:setTabDrawMode(editor.settings.tab.drawMode)


--caret
nmmStyle:setCaretStyle(editor.settings.caret.style)
nmmStyle:setBlinkCursor(editor.settings.caret.blink)
nmmStyle:setCaretFore(colors.caret.fore)

--
if editor.settings.caret.lineVisible then
    nmmStyle:setCaretLineBack(colors.caretLineBack.back)
end
nmmStyle:setCaretLineVisible(editor.settings.caret.lineVisible)

--whitespace
nmmStyle:setViewWS(editor.settings.whitespace.flags)
nmmStyle:setWhitespaceFore(colors.whitespace.fore)
nmmStyle:setWhitespaceSize(editor.settings.whitespace.size)

--EOL
local eolMode = nmmStyle:detectEOLMode()
nmmStyle:setEOLMode(eolMode)
nmmStyle:setViewEOL(editor.settings.eol)

--If the buffer size is greater than editor.settings.wrap.maxsize, it is forced to WRAP_NONE
--wrap mode
if nmmStyle:getBufferSize() > editor.settings.wrap.maxsize then
    nmmStyle:setWrapMode(NmmStyle.SC_WRAP_NONE)
else
    nmmStyle:setWrapMode(editor.settings.wrap.style)
end
nmmStyle:setWrapVisualFlags(editor.settings.wrap.visualFlags)

--enable auto completion
nmmStyle:setAutoCompletion(editor.settings.autoCompletion.enable and nmmStyle:getBufferSize() <= editor.settings.autoCompletion.maxsize)

--layout cache
nmmStyle:setLayoutCache(editor.settings.cacheLayout)

--fold
nmmStyle:setFoldFlags(editor.settings.fold.flags)
nmmStyle:setFold(editor.settings.fold.enable)
nmmStyle:setFoldSymbol(editor.settings.fold.symbols, colors.fold.fore, colors.fold.back,
    colors.foldHighlightColor)

--selection
nmmStyle:setSelFore(editor.settings.selection.usefore, colors.selection.fore)
nmmStyle:setSelBack(true, colors.selection.back)

--indentation
nmmStyle:setAutoIndent(editor.settings.indent.auto)
nmmStyle:setUseTabs(editor.settings.indent.usetabs)
nmmStyle:setIndent(editor.settings.indent.size)
nmmStyle:setBackspaceUnIndents(editor.settings.indent.backspaceUnindents)
nmmStyle:setIndentationGuides(editor.settings.indent.guides)
nmmStyle:setIndentStartWords(editor.settings.indent.starts)
nmmStyle:setIndentEndWords(editor.settings.indent.ends)

--marker
nmmStyle:setIndicatorComposingFore(colors.indicatorComposing.fore)
nmmStyle:setIndicatorFindMarkerFore(colors.indicatorFindmarker.fore)

--fast scroller style
nmmStyle:loadFastScrollerStyle()

--show line number
nmmStyle:setShowLineNumber(editor.settings.lineNumber)

-- use volume key
nmmStyle:setUseVolumeKey(editor.settings.useVolumeKey)
-- scale gesture
nmmStyle:setUseScaleGesture(editor.settings.useScaleGesture)


nmmStyle:configEnd()



return lexer.lang

